@@ -188,7 +188,6 @@ namespace Docky.Interface
 		/// </summary>
 		double remove_index;
 		int remove_size;
-		int maxIconSize;
 		
 		uint animation_timer;
 		uint icon_size_timer;
@@ -230,6 +229,7 @@ namespace Docky.Interface
 				RegisterPreferencesEvents (value);
 				
 				// Initialize value
+				MaxIconSize = Preferences.IconSize;
 				UpdateMaxIconSize ();
 			}
 		}
@@ -259,9 +259,13 @@ namespace Docky.Interface
 				DragTracker.RepositionMode = config_mode;
 				if (value)
 					DragTracker.DragDisabled = false;
-				update_screen_regions = true;
+				UpdateScreenRegions ();
 				
 				SetTooltipVisibility ();
+				if (background_buffer != null) {
+					background_buffer.Dispose ();
+					background_buffer = null;
+				}
 				AnimatedDraw ();
 			}
 		}
@@ -276,7 +280,7 @@ namespace Docky.Interface
 		public ReadOnlyCollection<AbstractDockItem> Items {
 			get {
 				if (collection_backend.Count == 0) {
-					update_screen_regions = true;
+					UpdateScreenRegions ();
 					bool priorItems = false;
 					bool separatorNeeded = false;
 					
@@ -360,6 +364,8 @@ namespace Docky.Interface
 				Pango.CairoHelper.LayoutPath (surface.Context, layout);
 				surface.Context.Color = HoverTextManager.IsLight ? new Cairo.Color (0.1, 0.1, 0.1) : new Cairo.Color (1, 1, 1);
 				surface.Context.Fill ();
+				
+				layout.Context.Dispose ();
 			}
 			
 			return surface;
@@ -379,8 +385,9 @@ namespace Docky.Interface
 			} else if (hoveredItem != null && background_buffer != null) {
 				if (ExternalDragActive) {
 					if (DragTracker.ItemAcceptsDrop ()) {
-						if (launch_hover_buffer == null)
-							launch_hover_buffer = DrawHoverText (launch_hover_buffer, Catalog.GetString ("Drop to open"));
+						if (launch_hover_buffer != null)
+							launch_hover_buffer.Dispose ();
+						launch_hover_buffer = DrawHoverText (launch_hover_buffer, hoveredItem.DropText);
 						hover = launch_hover_buffer;
 					} else if (DragTracker.ProviderAcceptsDrop ()) {
 						if (drop_hover_buffer == null)
@@ -499,10 +506,7 @@ namespace Docky.Interface
 			get { return Math.Min (MaxIconSize, Preferences.IconSize); }
 		}
 		
-		int MaxIconSize {
-			get { return Math.Min (Preferences.IconSize, maxIconSize); }
-			set { maxIconSize = value; } 
-		}
+		int MaxIconSize { get; set; }
 		
 		int Monitor {
 			get { return Preferences.MonitorNumber; }
@@ -588,13 +592,15 @@ namespace Docky.Interface
 					DockWidth - PainterBufferSize, 
 					DockHeight - 2 * DockWidthBuffer);
 				
+				int maxWidth = VerticalDock ? monitor_geo.Height : monitor_geo.Width;
+				
 				if (Painter.Allocation != allocation) {
 					Painter.SetAllocation (allocation);
-					allocation.Width = Painter.MinimumWidth;
+					allocation.Width = Math.Min (maxWidth - PainterBufferSize, Painter.MinimumWidth);
 					Painter.SetAllocation (allocation);
 				}
 				
-				last_painter_width = Painter.MinimumWidth + PainterBufferSize;
+				last_painter_width = Math.Min (maxWidth, Painter.MinimumWidth + PainterBufferSize);
 				return last_painter_width;
 			}
 		}
@@ -659,16 +665,14 @@ namespace Docky.Interface
 				
 				// we buffer this value during renders since it will be checked many times and we dont need to 
 				// recalculate it each time
-				if (zoom_in_buffer.HasValue && rendering) {
+				if (zoom_in_buffer.HasValue && rendering)
 					return zoom_in_buffer.Value;
-				}
 				
 				double zoom = Math.Min (1, (render_time - dock_hovered_change_time).TotalMilliseconds / 
 				                        BaseAnimationTime.TotalMilliseconds);
 				
-				if (!DockHovered) {
+				if (!DockHovered)
 					zoom = 1 - zoom;
-				}
 				
 				zoom *= 1 - PainterOpacity;
 				
@@ -678,13 +682,6 @@ namespace Docky.Interface
 				return zoom;
 			}
 		}
-		
-		int ZoomSize {
-			get { 
-				// 330 chosen for its pleasant (to me) look
-				return (int) (330 * (IconSize / 64.0)); 
-			}
-		}
 
 		#endregion
 		
@@ -730,6 +727,7 @@ namespace Docky.Interface
 			
 			Realized                      += HandleRealized;
 			Docky.Controller.ThemeChanged += DockyControllerThemeChanged;
+			Wnck.Screen.Default.WindowOpened += HandleWindowOpened;
 			
 			// fix for nvidia bug
 			if (Docky.CommandLinePreferences.BufferTime > 0)
@@ -769,7 +767,11 @@ namespace Docky.Interface
 				if (now - adi.StateSetTime (ItemState.Move) < SlideTime)
 					return true;
 				//Bouncing Items
-				if ((now - adi.LastClick) < BounceTime || (now - adi.StateSetTime (ItemState.Urgent)) < BounceTime)
+				if ((now - adi.LastClick) < BounceTime)
+					return true;
+				if ((adi.State & ItemState.Urgent) != ItemState.Urgent)
+					continue;
+				if ((now - adi.StateSetTime (ItemState.Urgent)) < BounceTime)
 					return true;
 				//Glowing Items
 				if ((now - adi.StateSetTime (ItemState.Urgent)) < GlowTime)
@@ -780,14 +782,16 @@ namespace Docky.Interface
 		
 		void HandleMenuHidden (object sender, EventArgs e)
 		{
+			UpdateScreenRegions ();
 			SetTooltipVisibility ();
 			AnimatedDraw ();
 		}
 
 		void HandleMenuShown (object sender, EventArgs e)
 		{
-			AnimatedDraw ();
+			UpdateScreenRegions ();
 			SetTooltipVisibility ();
+			AnimatedDraw ();
 		}
 
 		void DockyControllerThemeChanged (object sender, EventArgs e)
@@ -1196,11 +1200,31 @@ namespace Docky.Interface
 				else
 					Painter.ButtonReleased (x, y, evnt.State);
 			} else if (HoveredItem != null && lastClickedItem == HoveredItem) {
-				double x, y;
 				Gdk.Rectangle region = DrawRegionForItem (HoveredItem);
 					
-				x = ((Cursor.X + window_position.X) - region.X) / (double) region.Height;
-				y = ((Cursor.Y + window_position.Y) - region.Y) / (double) region.Width;
+				double x;
+				double y = Math.Min (1, Math.Max (0, (Cursor.Y - window_position.Y - region.Y) / (double) region.Height));
+				
+				if (VerticalDock)
+					x = Math.Min (1, Math.Max (0, (Cursor.X - window_position.X - region.X) / (double) region.Width));
+				else
+					x = Math.Min (1, Math.Max (0, (Cursor.X + window_position.X - region.X) / (double) region.Width));
+				
+				if (HoveredItem.RotateWithDock) {
+					if (Position == DockPosition.Top) {
+						x = 1 - x;
+						y = 1 - y;
+					} else if (VerticalDock) {
+						double tmp = x;
+						x = y;
+						y = tmp;
+						
+						if (Position == DockPosition.Left)
+							y = 1 - y;
+						else
+							x = 1 - x;
+					}
+				}
 				
 				HoveredItem.Clicked (evnt.Button, evnt.State, x, y);
 				AnimatedDraw ();
@@ -1231,6 +1255,12 @@ namespace Docky.Interface
 		#endregion
 		
 		#region Misc.
+		void HandleWindowOpened (object o, WindowOpenedArgs args)
+		{
+			UpdateScreenRegions ();
+			AnimatedDraw ();
+		}
+		
 		void Reconfigure ()
 		{
 			SetSizeRequest ();
@@ -1266,19 +1296,23 @@ namespace Docky.Interface
 		internal void UpdateCollectionBuffer ()
 		{
 			if (rendering) {
-				// resetting a durring a render is bad. Complete the render then reset.
+				// resetting during a render is bad. Complete the render, then reset.
 				GLib.Idle.Add (delegate {
 					// dispose of our separators as we made them ourselves,
 					// this could be a bit more elegant
-					foreach (AbstractDockItem item in Items.Where (adi => adi is INonPersistedItem))
+					foreach (AbstractDockItem item in Items.Where (adi => adi is INonPersistedItem && adi != DockyItem)) {
+						DrawValues.Remove (item);
 						item.Dispose ();
+					}
 					
 					collection_backend.Clear ();
 					return false;
 				});
 			} else {
-				foreach (AbstractDockItem item in Items.Where (adi => adi is INonPersistedItem))
+				foreach (AbstractDockItem item in Items.Where (adi => adi is INonPersistedItem && adi != DockyItem)) {
+					DrawValues.Remove (item);
 					item.Dispose ();
+				}
 				
 				collection_backend.Clear ();
 			}
@@ -1360,7 +1394,7 @@ namespace Docky.Interface
 			Painter.PaintNeeded += HandlePainterPaintNeeded;
 			
 			repaint_painter = true;
-			update_screen_regions = true;
+			UpdateScreenRegions ();
 			DragTracker.DragDisabled = true;
 			
 			SetTooltipVisibility ();
@@ -1380,7 +1414,7 @@ namespace Docky.Interface
 			Painter.PaintNeeded -= HandlePainterPaintNeeded;
 			
 			DragTracker.DragDisabled = false;
-			update_screen_regions = true;
+			UpdateScreenRegions ();
 			
 			Painter.Hidden ();
 			Painter = null;
@@ -1528,6 +1562,11 @@ namespace Docky.Interface
 		}
 		
 		Gdk.Rectangle DrawRegionForItemValue (AbstractDockItem item, DrawValue val)
+		{
+			return DrawRegionForItemValue (item, val, false);
+		}
+		
+		Gdk.Rectangle DrawRegionForItemValue (AbstractDockItem item, DrawValue val, bool hoverRegion)
 		{
 			int width = IconSize, height = IconSize;
 			
@@ -1535,7 +1574,7 @@ namespace Docky.Interface
 				DockySurface surface = item.IconSurface (main_buffer, IconSize, IconSize, VisibleDockHeight);
 				
 				width = surface.Width;
-					height = surface.Height;
+				height = surface.Height;
 				
 				if (item.RotateWithDock && VerticalDock) {
 					int tmp = width;
@@ -1544,6 +1583,18 @@ namespace Docky.Interface
 				}
 			}
 			
+			if (hoverRegion)
+				if (VerticalDock)
+					return new Gdk.Rectangle ((int) (val.Center.X - (width * val.Zoom / 2)),
+						(int) (val.StaticCenter.Y - height / 2),
+						(int) (width * val.Zoom),
+						height);
+				else
+					return new Gdk.Rectangle ((int) (val.StaticCenter.X - width / 2),
+						(int) (val.Center.Y - (height * val.Zoom / 2)),
+						width,
+						(int) (height * val.Zoom));
+			
 			return new Gdk.Rectangle ((int) (val.Center.X - (width * val.Zoom / 2)),
 				(int) (val.Center.Y - (height * val.Zoom / 2)),
 				(int) (width * val.Zoom),
@@ -1561,52 +1612,40 @@ namespace Docky.Interface
 			// first we do the math as if this is a top dock, to do this we need to set
 			// up some "pretend" variables. we pretend we are a top dock because 0,0 is
 			// at the top.
-			int width;
-			int height;
+			int width = surface.Width;
+			int height = surface.Height;
 			double zoom;
 			
-			// our width and height switch around if we have a veritcal dock
-			if (!VerticalDock) {
-				width = surface.Width;
-				height = surface.Height;
-			} else {
-				width = surface.Height;
-				height = surface.Width;
-			}
 			
 			Gdk.Point cursor = LocalCursor;
 			Gdk.Point localCursor = cursor;
 			
 			// "relocate" our cursor to be on the top
 			switch (Position) {
-			case DockPosition.Top:
-				;
-				break;
-			case DockPosition.Left:
-				int tmpY = cursor.Y;
-				cursor.Y = cursor.X;
-				cursor.X = width - (width - tmpY);
-				break;
 			case DockPosition.Right:
-				tmpY = cursor.Y;
 				cursor.X = (Width - 1) - cursor.X;
-				cursor.Y = cursor.X;
-				cursor.X = width - (width - tmpY);
 				break;
 			case DockPosition.Bottom:
 				cursor.Y = (Height - 1) - cursor.Y;
 				break;
+			default:
+				break;
 			}
 			
-			// the line along the dock width about which the center of unzoomed icons sit
-			int midline = DockHeight / 2;
-			
-			// the left most edge of the first dock item
-			int startX = ((width - DockWidth) / 2) + DockWidthBuffer;
+			if (VerticalDock) {
+				int tmpY = cursor.Y;
+				cursor.Y = cursor.X;
+				cursor.X = tmpY;
+				
+				// our width and height switch around if we have a veritcal dock
+				width = surface.Height;
+				height = surface.Width;
+			}
 			
 			// this offset is used to split the icons into left/right aligned for panel mode
-			int panel_item_offset = 0;
 			double panelanim = PanelModeToggleProgress;
+			int panel_item_offset;
+			
 			if (VerticalDock)
 				panel_item_offset = (monitor_geo.Height - DockWidth) / 2;
 			else
@@ -1619,206 +1658,216 @@ namespace Docky.Interface
 				if (Preferences.PanelMode)
 					panel_item_offset = (int) (panel_item_offset * panelanim);
 				else
-					panel_item_offset = panel_item_offset - (int) (panel_item_offset * panelanim);
+					panel_item_offset -= (int) (panel_item_offset * panelanim);
 			}
 			
+			// the line along the dock width about which the center of unzoomed icons sit
+			int midline = DockHeight / 2;
+			
+			// the left most edge of the first dock item
+			int startX = ((width - DockWidth) / 2) + DockWidthBuffer - panel_item_offset;
+			
 			Gdk.Point center = new Gdk.Point (startX, midline);
 			
-			int index = 0;
+			// right align docklets
 			bool rightAlign = (Items [0].Owner != Preferences.DefaultProvider && Items [0] != DockyItem);
+			if (rightAlign)
+				center.X += 2 * panel_item_offset;
+			
+			int index = 0;
 			foreach (AbstractDockItem adi in Items) {
-				if (adi is SeparatorItem)
-					rightAlign = true;
+				// anything after the first separator is a docklet, and right aligned
+				if (!rightAlign && adi is SeparatorItem) {
+ 					rightAlign = true;
+					center.X += 2 * panel_item_offset;
+				}
 				
-				// used to handle remove animation
+				// used to handle remove animations
 				if (remove_index != 0 && remove_index < index && remove_index > index - 1) {
 					double removePercent = 1 - Math.Min (1, (DateTime.UtcNow - remove_time).TotalMilliseconds / BaseAnimationTime.TotalMilliseconds);
-					if (removePercent == 0) {
+					if (removePercent == 0)
 						remove_index = 0;
-					} else {
+					else
 						center.X += (int) ((remove_size + ItemWidthBuffer) * removePercent);
-					}
 				}
 				
 				DrawValue val = new DrawValue ();
 				int iconSize = IconSize;
 				
 				// div by 2 may result in rounding errors? Will this render OK? Shorts WidthBuffer by 1?
-				double halfSize;
-				if (adi.Square) {
-					halfSize = iconSize / 2.0;
-				} else {
-					DockySurface icon = adi.IconSurface (surface, iconSize, IconSize, VisibleDockHeight);
-					
-					// yeah I am pretty sure...
-					if (adi.RotateWithDock || !VerticalDock) {
-						halfSize = icon.Width / 2.0;
-					} else {
-						halfSize = icon.Height / 2.0;
-					}
+				double halfSize = iconSize / 2.0;
+ 				
+				if (!adi.Square) {
+					DockySurface icon = adi.IconSurface (surface, iconSize, iconSize, VisibleDockHeight);
+					halfSize = ((adi.RotateWithDock || !VerticalDock) ? icon.Width : icon.Height) / 2.0;
 				}
 				
+				// animate adding new icon
 				halfSize *= Math.Min (1, (DateTime.UtcNow - adi.AddTime).TotalMilliseconds / BaseAnimationTime.TotalMilliseconds);
 				
 				// center now represents our midpoint
 				center.X += (int) Math.Floor (halfSize);
 				val.StaticCenter = new PointD (center.X, center.Y);
 				
-				if (ZoomPercent > 1) {
-					// get us some handy doubles with fancy names
-					double cursorPosition = cursor.X;
-					double centerPosition = center.X;
-					
-					// ZoomPercent is a number greater than 1.  It should never be less than one.
-					// ZoomIn is a range of 0 to 1. we need a number that is 1 when ZoomIn is 0, 
-					// and ZoomPercent when ZoomIn is 1.  Then we treat this as 
-					// if it were the ZoomPercent for the rest of the calculation
-					double zoomInPercent = 1 + (ZoomPercent - 1) * ZoomIn;
-					
-					// offset from the center of the true position, ranged between 0 and half of the zoom range
-					double offset = Math.Min (Math.Abs (cursorPosition - centerPosition), ZoomSize / 2);
-					
-					double offsetPercent;
-					if (ExternalDragActive) {
-						// Provide space for dropping between items
-						offset += ZoomedIconSize * (offset / (ZoomSize / 2.0));
-						offsetPercent = Math.Min (1, offset / (ZoomSize / 2.0 + ZoomedIconSize));
-					} else {
-						offsetPercent = offset / (ZoomSize / 2.0);
-					}
-					
-					if (offsetPercent > .99)
-						offsetPercent = 1;
+				// get us some handy doubles with fancy names
+				double cursorPosition = cursor.X;
+				double centerPosition = center.X;
+				
+				// ZoomPercent is a number greater than 1.  It should never be less than one.
+				
+				// zoomInPercent is a range of 1 to ZoomPercent.
+				// We need a number that is 1 when ZoomIn is 0, and ZoomPercent when ZoomIn is 1.
+				// Then we treat this as if it were the ZoomPercent for the rest of the calculation.
+				double zoomInPercent = 1 + (ZoomPercent - 1) * ZoomIn;
+				
+				double zoomSize = ZoomEnabled ? ZoomedIconSize : 2.0 * IconSize;
+				
+				// offset from the center of the true position, ranged between 0 and the zoom size
+				double offset = Math.Min (Math.Abs (cursorPosition - centerPosition), zoomSize);
+				
+				double offsetPercent;
+				if (ExternalDragActive) {
+					// Provide space for dropping between items
+					offset += offset * zoomSize / IconSize;
+					offsetPercent = Math.Min (1, offset / (zoomSize + ZoomedIconSize));
+				} else {
+					offsetPercent = offset / zoomSize;
+				}
+				
+				if (offsetPercent > .99)
+					offsetPercent = 1;
+				
+				// pull in our offset to make things less spaced out
+				// explaination since this is a bit tricky...
+				// we have three terms, basically offset = f(x) * h(x) * g(x)
+				// f(x) == offset identity
+				// h(x) == a number from 0 to DockPreference.ZoomPercent - 1.  This is used to get the smooth "zoom in" effect.
+				//         additionally serves to "curve" the offset based on the max zoom
+				// g(x) == a term used to move the ends of the zoom inward.  Precalculated that the edges should be 66% of the current
+				//         value. The center is 100%. (1 - offsetPercent) == 0,1 distance from center
+				// The .66 value comes from the area under the curve.  Dont ask me to explain it too much because it's too clever for me.
+				
+				// for external drags with no zoom, we pretend there is actually a zoom of 200%
+				if (ExternalDragActive && ZoomPercent == 1)
+					offset *= ZoomIn / 2.0;
+				else
+					offset *= zoomInPercent - 1;
+				offset *= 1 - offsetPercent / 3;
+				
+				if (cursorPosition > centerPosition)
+					centerPosition -= offset;
+				else
+					centerPosition += offset;
+				
+				if (!adi.Zoom) {
+					val.Zoom = 1;
+					val.Center = new Cairo.PointD ((int) centerPosition, center.Y);
+				} else {
 					// zoom is calculated as 1 through target_zoom (default 2).  
 					// The larger your offset, the smaller your zoom
 					
-					// First we get the point on our curve that defines out current zoom
+					// First we get the point on our curve that defines our current zoom
 					// offset is always going to fall on a point on the curve >= 0
 					zoom = 1 - Math.Pow (offsetPercent, 2);
 					
-					// scale this to match out zoomInPercent
+					// scale this to match our zoomInPercent
 					zoom = 1 + zoom * (zoomInPercent - 1);
 					
-					// pull in our offset to make things less spaced out
-					// explaination since this is a bit tricky...
-					// we have three terms, basically offset = f(x) * h(x) * g(x)
-					// f(x) == offset identify
-					// h(x) == a number from 0 to DockPreference.ZoomPercent - 1.  This is used to get the smooth "zoom in" effect.
-					//         additionally serves to "curve" the offset based on the max zoom
-					// g(x) == a term used to move the ends of the zoom inward.  Precalculated that the edges should be 66% of the current
-					//         value. The center is 100%. (1 - offsetPercent) == 0,1 distance from center
-					// The .66 value comes from the area under the curve.  Dont as me to explain it too much because it's too clever for me
-					offset = offset * (zoomInPercent - 1) * (1 - offsetPercent / 3);
+					double zoomedCenterHeight = DockHeightBuffer + (iconSize * zoom / 2.0);
 					
+					if (zoom == 1)
+						centerPosition = Math.Round (centerPosition);
 					
-					if (cursorPosition > centerPosition) {
-						centerPosition -= offset;
-					} else {
-						centerPosition += offset;
-					}
-					
-					if (!adi.Zoom) {
-						val.Zoom = 1;
-						val.Center = new Cairo.PointD ((int) centerPosition, center.Y);
-					} else {
-						double zoomedCenterHeight = DockHeightBuffer + (iconSize * zoom / 2.0);
-						
-						if (zoom == 1)
-							centerPosition = Math.Round (centerPosition);
-						
-						val.Center = new Cairo.PointD (centerPosition, zoomedCenterHeight);
-						val.Zoom = zoom;
-					}
-				} else {
-					val.Zoom = 1;
-					val.Center = new PointD (center.X, center.Y);
+					val.Center = new Cairo.PointD (centerPosition, zoomedCenterHeight);
+					val.Zoom = zoom;
 				}
 				
 				// now we undo our transforms to the point
-				switch (Position) {
-				case DockPosition.Top:
-					;
-					break;
-				case DockPosition.Left:
+				if (VerticalDock) {
 					double tmpY = val.Center.Y;
 					val.Center.Y = val.Center.X;
-					val.Center.X = width - (width - tmpY);
+					val.Center.X = tmpY;
 					
 					tmpY = val.StaticCenter.Y;
 					val.StaticCenter.Y = val.StaticCenter.X;
-					val.StaticCenter.X = width - (width - tmpY);
-					break;
+					val.StaticCenter.X = tmpY;
+				}
+				
+				switch (Position) {
 				case DockPosition.Right:
-					tmpY = val.Center.Y;
-					val.Center.Y = val.Center.X;
-					val.Center.X = width - (width - tmpY);
 					val.Center.X = (height - 1) - val.Center.X;
-					
-					tmpY = val.StaticCenter.Y;
-					val.StaticCenter.Y = val.StaticCenter.X;
-					val.StaticCenter.X = width - (width - tmpY);
 					val.StaticCenter.X = (height - 1) - val.StaticCenter.X;
 					break;
 				case DockPosition.Bottom:
 					val.Center.Y = (height - 1) - val.Center.Y;
 					val.StaticCenter.Y = (height - 1) - val.StaticCenter.Y;
 					break;
-				}
-				
-				// split the icons into left/right aligned for panel mode
-				switch (Position) {
 				default:
-				case DockPosition.Left:
-				case DockPosition.Top:
-					val = val.MoveRight (Position, rightAlign ? panel_item_offset : -panel_item_offset);
-					break;
-				case DockPosition.Right:
-				case DockPosition.Bottom:
-					val = val.MoveRight (Position, rightAlign ? -panel_item_offset : panel_item_offset);
 					break;
 				}
 				
-				Gdk.Rectangle hoverArea = DrawRegionForItemValue (adi, val);
+				Gdk.Rectangle hoverArea = DrawRegionForItemValue (adi, val, true);
 				
 				if (VerticalDock) {
 					hoverArea.Inflate ((int) (ZoomedDockHeight * .3), ItemWidthBuffer / 2);
+					hoverArea.Width += DockHeightBuffer;
 				} else {
 					hoverArea.Inflate (ItemWidthBuffer / 2, (int) (ZoomedDockHeight * .3));
+					hoverArea.Height += DockHeightBuffer;
+				}
+				
+				switch (Position) {
+				case DockPosition.Right:
+					hoverArea.X -= DockHeightBuffer;
+					break;
+				case DockPosition.Bottom:
+					hoverArea.Y -= DockHeightBuffer;
+					break;
+				default:
+					break;
 				}
 				
 				val.HoverArea = hoverArea;
 				DrawValues[adi] = val;
 
-				if (hoverArea.Contains (localCursor) && !AutohideManager.Hidden) {
-					//keep the hovereditem in mind, but don't change it while rendering
+				// keep the hovereditem in mind, but don't change it while rendering
+				if (hoverArea.Contains (localCursor) && !AutohideManager.Hidden)
 					next_hoveredItem = adi;
-				}
 				
 				if (update_screen_regions) {
-					Gdk.Rectangle region = hoverArea;
-					region.X += window_position.X;
-					region.Y += window_position.Y;
-					if (ConfigurationMode || Painter != null)
+					if (Menu.Visible || ConfigurationMode || Painter != null) {
 						adi.SetScreenRegion (Screen, new Gdk.Rectangle (0, 0, 0, 0));
-					else
+					} else {
+						Gdk.Rectangle region = hoverArea;
+						region.X += window_position.X;
+						region.Y += window_position.Y;
 						adi.SetScreenRegion (Screen, region);
+					}
 				}
 				
 				// move past midpoint to end of icon
 				center.X += (int) Math.Ceiling (halfSize) + ItemWidthBuffer;
+				
 				index++;
 			}
 			
 			update_screen_regions = false;
 		}
 		
+		void UpdateScreenRegions ()
+		{
+			GLib.Timeout.Add (10 + (uint) Math.Max (BaseAnimationTime.TotalMilliseconds, SlideTime.TotalMilliseconds), delegate {
+				update_screen_regions = true;
+				AnimatedDraw ();
+				return false;
+			});
+		}
+		
 		void UpdateMaxIconSize ()
 		{
 			if (Painter != null)
 				return;
 			
-			MaxIconSize = Preferences.IconSize;
-			
 			if (icon_size_timer > 0)
 				GLib.Source.Remove (icon_size_timer);
 			
@@ -1835,11 +1884,12 @@ namespace Docky.Interface
 					}
 				} else if (MaxIconSize < Preferences.IconSize) {
 					// Perhaps MaxIconSize is too small, lets find out!
-					while (dockWidth < maxWidth && MaxIconSize <= Preferences.IconSize) {
+					while (dockWidth < maxWidth && MaxIconSize < Preferences.IconSize) {
 						MaxIconSize++;
 						dockWidth = DockWidth;
 					}
-					MaxIconSize--;
+				} else {
+					MaxIconSize = Preferences.IconSize;
 				}
 				AnimatedDraw ();
 				icon_size_timer = 0;
@@ -2066,9 +2116,9 @@ namespace Docky.Interface
 						DrawValue val = DrawValues [adi];
 						DrawValue glowloc;
 						if (preferences.FadeOnHide) {
-							glowloc = val.MoveIn (Position, IconSize * val.Zoom / 2 - DockHeight);
+							glowloc = val.MoveIn (Position, -urgent_glow_buffer.Height / 2 + DockHeightBuffer);
 						} else {
-							glowloc = val.MoveIn (Position, IconSize * val.Zoom / 2);
+							glowloc = val.MoveIn (Position, -urgent_glow_buffer.Height / 2 + DockHeightBuffer + ZoomedDockHeight);
 						}
 
 						double opacity = 0.2 + (0.75 * (Math.Sin ((render_time - adi.StateSetTime (ItemState.Urgent)).TotalMilliseconds / PulseTime.TotalMilliseconds * 2 * Math.PI) + 1) / 2);
@@ -2186,12 +2236,11 @@ namespace Docky.Interface
 				double slideProgress = (render_time - item.StateSetTime (ItemState.Move)).TotalMilliseconds / SlideTime.TotalMilliseconds;
 
 				double move = (item.Position - item.LastPosition) * (IconSize * val.Zoom + ItemWidthBuffer) 
-					//draw the anitmation backwards cause item has already moved
+					//draw the animation backwards cause item has already moved
 					* (1 - slideProgress);
                 
-				if (Position == DockPosition.Top || Position == DockPosition.Left) {
+				if (Position == DockPosition.Top || Position == DockPosition.Left)
 					move *= -1;
-				}
 				
 				val = val.MoveRight (Position, move);
 			}
@@ -2219,17 +2268,15 @@ namespace Docky.Interface
 				}
 			}
 			
-			if (HoveredAcceptsDrop && HoveredItem == item && ExternalDragActive) {
+			if (HoveredAcceptsDrop && HoveredItem == item && ExternalDragActive)
 				lighten += .4;
-			} else if (!ZoomEnabled && !Menu.Visible && HoveredItem == item && !ExternalDragActive && !InternalDragActive && !ConfigurationMode) {
+			else if (!ZoomEnabled && !Menu.Visible && HoveredItem == item && !ExternalDragActive && !InternalDragActive && !ConfigurationMode)
 				lighten += .2;
-			}
 			
-			if ((item.State & ItemState.Wait) != 0) {
+			if ((item.State & ItemState.Wait) != 0)
 				darken += .5;
-			} else if (Menu.Visible && HoveredItem == item) {
+			else if (Menu.Visible && HoveredItem == item)
 				darken += .4;
-			}
 			
 			if ((item.State & ItemState.Urgent) == ItemState.Urgent && 
 				(render_time - item.StateSetTime (ItemState.Urgent)) < BounceTime) {
@@ -2245,23 +2292,22 @@ namespace Docky.Interface
 			
 			double renderZoom = 1, renderRotation = 0;
 			
-			if (item.RotateWithDock) {
+			if (item.RotateWithDock)
 				switch (Position) {
 				case DockPosition.Top:
 					renderRotation = Math.PI;
 					break;
 				case DockPosition.Left:
-					renderRotation = Math.PI * 1.5;
+					renderRotation = Math.PI * .5;
 					break;
 				case DockPosition.Right:
-					renderRotation = Math.PI * .5;
+					renderRotation = Math.PI * 1.5;
 					break;
 				default:
 				case DockPosition.Bottom:
 					renderRotation = 0;
 					break;
 				}
-			}
 
 			if (item.Zoom && !(item.ScalableRendering && center.Zoom == 1)) {
 				icon = item.IconSurface (surface, ZoomedIconSize, IconSize, VisibleDockHeight);
@@ -2342,12 +2388,7 @@ namespace Docky.Interface
 				
 				DrawValue loc = val.MoveIn (Position, 1 - IconSize * val.Zoom / 2 - DockHeightBuffer);
 				
-				DockySurface indicator;
-				if ((item.State & ItemState.Urgent) == ItemState.Urgent) {
-					indicator = urgent_indicator_buffer;
-				} else {
-					indicator = normal_indicator_buffer;
-				}
+				DockySurface indicator = (item.State & ItemState.Urgent) == ItemState.Urgent ? urgent_indicator_buffer : normal_indicator_buffer;
 				
 				if (item.Indicator == ActivityIndicator.Single || !Preferences.IndicateMultipleWindows) {
 					indicator.ShowWithOptions (surface, loc.Center, 1, 0, 1);
@@ -2521,6 +2562,12 @@ namespace Docky.Interface
 					background_buffer = new DockySurface (BackgroundWidth, BackgroundHeight, surface);
 				}
 				
+				if (ConfigurationMode) {
+					background_buffer.Context.Rectangle (0, 0, BackgroundWidth, BackgroundHeight);
+					background_buffer.Context.Color = new Cairo.Color (1, 1, 1, 0.10);
+					background_buffer.Context.Fill ();
+				}
+				
 				Gdk.Pixbuf background = DockServices.Drawing.LoadIcon (Docky.Controller.BackgroundSvg);
 				Gdk.Pixbuf tmp;
 				
@@ -2551,7 +2598,7 @@ namespace Docky.Interface
 				background.Dispose ();
 			}
 			
-			double tilt = .6;
+			double tilt = .6 - (double) DockHeightBuffer / (double) backgroundArea.Height;
 			tilt *= 1 - PainterOpacity;
 			double tiltanim = Math.Min (1, ((rendering ? render_time : DateTime.UtcNow) - threedimensional_change_time).TotalMilliseconds / BaseAnimationTime.TotalMilliseconds);
 			tilt *= ThreeDimensional ? tiltanim : 1 - tiltanim;
@@ -2610,6 +2657,9 @@ namespace Docky.Interface
 				}
 				
 				cr.Paint ();
+
+				(cr.Target as IDisposable).Dispose ();
+
 				rendering = false;
 				
 				//now after rendering we can set the new HoveredItem
@@ -2709,10 +2759,13 @@ namespace Docky.Interface
 			AutohideManager.DockHoveredChanged  -= HandleDockHoveredChanged;
 			Screen.SizeChanged                  -= ScreenSizeChanged;
 			Docky.Controller.ThemeChanged       -= DockyControllerThemeChanged;
+			Wnck.Screen.Default.WindowOpened    -= HandleWindowOpened;
 			
 			// clear out our separators
-			foreach (AbstractDockItem adi in Items.Where (adi => adi is INonPersistedItem))
+			foreach (AbstractDockItem adi in Items.Where (adi => adi is INonPersistedItem && adi != DockyItem)) {
+				DrawValues.Remove (adi);
 				adi.Dispose ();
+			}
 			
 			ResetBuffers ();
 			
